@@ -45,7 +45,8 @@ use Config;
 use Carp;
 use File::Spec;
 use Data::Dumper;
-our $VERSION = '0.27'; # VERSION
+
+our $VERSION = '0.28'; # VERSION
 
 my $SIG_INT   = 0;
 my $SIG_PIPE  = 0;
@@ -57,85 +58,6 @@ my $SIG_ALARM = 0;
 
 =head1 ATTRIBUTES
 
-=head2 server
-
-This is a string representing the servername where the instance should connect. This is a optional attribute during
-object creation with the C<new> method.
-
-Beware that the C<run> method will verify if the C<server> attribute has a defined value or not: if it has, the C<run>
-method will try to connect to the Siebel Enterprise specifying the given Siebel Server. If not, the method will try to connect
-to the Enterprise only, not specifying which Siebel Server to connect.
-
-=cut
-
-has server => (
-    isa      => 'NotNullStr',
-    is       => 'rw',
-    required => 0,
-    reader   => 'get_server',
-    writer   => 'set_server'
-);
-
-=head2 gateway
-
-This is a string representing the gateway where the instance should connect. This is a required attribute during
-object creation with the C<new> method.
-
-=cut
-
-has gateway => (
-    isa      => 'NotNullStr',
-    is       => 'rw',
-    required => 1,
-    reader   => 'get_gateway',
-    writer   => 'set_gateway'
-);
-
-=head2 enterprise
-
-This is a string representing the enterprise where the instance should connect. This is a required attribute during
-object creation with the C<new> method.
-
-=cut
-
-has enterprise => (
-    isa      => 'NotNullStr',
-    is       => 'rw',
-    required => 1,
-    reader   => 'get_enterprise',
-    writer   => 'set_enterprise'
-);
-
-=head2 user
-
-This is a string representing the login for authentication. This is a required attribute during
-object creation with the C<new> method.
-
-=cut
-
-has user => (
-    isa      => 'NotNullStr',
-    is       => 'rw',
-    required => 1,
-    reader   => 'get_user',
-    writer   => 'set_user'
-);
-
-=head2 password
-
-This is a string representing the password for authentication. This is a required attribute during
-object creation with the C<new> method.
-
-=cut
-
-has password => (
-    isa      => 'NotNullStr',
-    is       => 'rw',
-    required => 1,
-    reader   => 'get_password',
-    writer   => 'set_password'
-);
-
 =head2 commands
 
 An array reference containing one or more references of L<Siebel::Srvrmgr::Daemon::Commands> class.
@@ -157,24 +79,6 @@ has commands => (
 
 =pod
 
-=head2 bin
-
-An string representing the full path to the C<srvrmgr> program in the filesystem.
-
-This is a required attribute during object creation with the C<new> method.
-
-=cut
-
-has bin => (
-    isa      => 'NotNullStr',
-    is       => 'rw',
-    required => 1,
-    reader   => 'get_bin',
-    writer   => 'set_bin'
-);
-
-=pod
-
 =head2 time_zone
 
 Required attribute.
@@ -226,20 +130,6 @@ It defaults to false.
 has use_perl =>
   ( isa => 'Bool', is => 'ro', reader => 'use_perl', default => 0 );
 
-=head2 lang_id
-
-A string representing the LANG_ID parameter to connect to srvrmgr. If defaults to "ENU";
-
-=cut
-
-has lang_id => (
-    isa     => 'Str',
-    is      => 'rw',
-    reader  => 'get_lang_id',
-    writer  => 'set_lang_id',
-    default => 'ENU'
-);
-
 =head2 child_runs
 
 An integer representing the number of times the child object was used in C<run> invocations. This is reset to zero if a new child process is created.
@@ -254,36 +144,6 @@ has child_runs => (
     default => 0
 );
 
-=head2 maximum_retries
-
-The maximum times this class wil retry to launch a new process of srvrmgr if the previous one failed for any reason. This is intented to implement
-robustness to the process.
-
-=cut
-
-has maximum_retries => (
-    isa     => 'Int',
-    is      => 'ro',
-    reader  => 'get_max_retries',
-    writer  => '_set_max_retries',
-    default => 5
-);
-
-=head2 retries
-
-The number of retries of launching a new srvrmgr process. If this value reaches the value defined for C<maximum_retries>, the instance of Siebel::Srvrmgr::Daemon
-will quit execution returning an error code.
-
-=cut
-
-has retries => (
-    isa     => 'Int',
-    is      => 'ro',
-    reader  => 'get_retries',
-    writer  => '_set_retries',
-    default => 0
-);
-
 =head2 clear_raw
 
 A boolean attribute that defines if the raw data recovered from C<srvrmgr> should be kept or discarded as soon as possibly.
@@ -300,18 +160,6 @@ has clear_raw => (
     default => 1
 );
 
-=head2 field_delimiter
-
-This is a single character attribute. It tells the Daemon class to consider a field delimiter, if such options was
-set in the C<srvrmgr> program. If this option is used but this attribute is not set accordinly, parsing will probably
-fail.
-
-Since this attribute should be defined during Daemon object instance, it is read-only.
-
-=cut
-
-has field_delimiter => ( is => 'ro', isa => 'Chr', reader => 'get_field_del' );
-
 =head2 has_lock
 
 Optional parameter.
@@ -386,10 +234,6 @@ Returns the content of the attribute C<time_zone>.
 
 Returns the content of the attribute C<cmd_stack>.
 
-=head2 get_field_del
-
-Getter for the C<field_delimiter> attribute.
-
 =head2 clear_raw
 
 Getter for the C<clear_raw> attribute.
@@ -422,54 +266,6 @@ Returns the value of the attribute C<ipc_buffer_size>.
 
 Sets the attribute C<ipc_buffer_size>. Expects an integer as parameter, multiple of 1024.
 
-=head2 get_lang_id
-
-Returns the value of the attribute C<lang_id>.
-
-=head2 set_lang_id
-
-Sets the attribute C<lang_id>. Expects a string as parameter.
-
-=head2 get_server
-
-Returns the content of C<server> attribute as a string.
-
-=head2 set_server
-
-Sets the attribute C<server>. Expects an string as parameter.
-
-=head2 get_gateway
-
-Returns the content of C<gateway> attribute as a string.
-
-=head2 set_gateway
-
-Sets the attribute C<gateway>. Expects a string as parameter.
-
-=head2 get_enterprise
-
-Returns the content of C<enterprise> attribute as a string.
-
-=head2 set_enterprise
-
-Sets the C<enterprise> attribute. Expects a string as parameter.
-
-=head2 get_user
-
-Returns the content of C<user> attribute as a string.
-
-=head2 set_user
-
-Sets the C<user> attribute. Expects a string as parameter.
-
-=head2 get_password
-
-Returns the content of C<password> attribute as a string.
-
-=head2 set_password
-
-Sets the C<password> attribute. Expects a string as parameter.
-
 =head2 get_commands
 
 Returns the content of the attribute C<commands>.
@@ -478,14 +274,6 @@ Returns the content of the attribute C<commands>.
 
 Set the content of the attribute C<commands>. Expects an array reference as parameter.
 
-=head2 get_bin
-
-Returns the content of the C<bin> attribute.
-
-=head2 set_bin
-
-Sets the content of the C<bin> attribute. Expects a string as parameter.
-
 =head2 get_pid
 
 Returns the content of C<pid> attribute as an integer.
@@ -517,9 +305,7 @@ If the logging is set to C<DEBUG> level all attribute values will be logged.
 =cut
 
 sub BUILD {
-
     my $self = shift;
-
     $SIG{INT}  = sub { $SIG_INT   = 1 };
     $SIG{PIPE} = sub { $SIG_PIPE  = 1 };
     $SIG{ALRM} = sub { $SIG_ALARM = 1 };
@@ -528,26 +314,20 @@ sub BUILD {
     my $logger = Siebel::Srvrmgr->gimme_logger( blessed($self) );
 
     if ( $logger->is_debug() ) {
-
         $logger->debug('This instance attributes values:');
 
         foreach my $attrib ( $self->meta->get_all_attributes() ) {
-
             my $getter = $attrib->get_read_method();
             my $value  = $self->$getter;
             $value = 'UNDEFINED' unless ( defined($value) );
 
             if ( ref($value) ne '' ) {
-
                 $logger->debug(
                     'attribute ' . $attrib->name() . ' = ' . Dumper($value) );
-
             }
             else {
-
                 $logger->debug(
                     'attribute ' . $attrib->name() . ' = ' . $value );
-
             }
 
         }
@@ -556,19 +336,25 @@ sub BUILD {
 
 }
 
-=head2 reset_retries
+=head2 cmds_vs_tree
 
-Reset the retries of creating a new process of srvrmgr program, setting the attribute C<retries> to zero.
+This method compares the number of C<commands> defined in a instance of this class with the number of nodes passed as parameter.
+
+If their are equal, the number is returned. If their are different (and there is a problem with the parsed output of srvrmgr) this method
+returns C<undef>.
 
 =cut
 
-sub reset_retries {
+sub cmds_vs_tree {
+    my ( $self, $nodes_num ) = @_;
+    my $cmds_num = scalar( @{ $self->get_commands() } );
 
-    my $self = shift;
-
-    $self->_set_retries(0);
-
-    return 1;
+    if ( $cmds_num == $nodes_num ) {
+        return $nodes_num;
+    }
+    else {
+        return;
+    }
 
 }
 
@@ -605,22 +391,16 @@ This method is also used internally through the C<_setup_commands> method.
 =cut
 
 sub check_cmd {
-
-    my $self = shift;
-    my $cmd  = shift;
-
+    my ( $self, $cmd ) = @_;
     confess( 'Invalid command received for execution: '
           . Dumper( $self->get_cmd_stack() ) )
       unless ( defined($cmd) );
-
     confess("Insecure command from command stack [$cmd]. Execution aborted")
       unless ( ( $cmd =~ /^load/ )
         or ( $cmd =~ /^list/ )
         or ( $cmd =~ /^set\sdelimiter\s[[:graph:]]/ )
         or ( $cmd =~ /^exit/ ) );
-
     return 1;
-
 }
 
 =pod
@@ -638,23 +418,17 @@ again.
 =cut
 
 sub shift_command {
-
-    my $self = shift;
-
+    my $self     = shift;
     my $cmds_ref = $self->get_commands();
 
     if ( scalar( @{$cmds_ref} ) > 1 ) {
-
         my $shifted = shift( @{$cmds_ref} );
         $self->set_commands($cmds_ref);    # must trigger the attribute
         return $shifted;
-
     }
     else {
-
         $self->set_commands( [] );         # must trigger the attribute
         return;
-
     }
 
 }
@@ -670,14 +444,12 @@ Expects as parameter a L<Siebel::Srvrmgr::Daemon::Command> instance.
 =cut
 
 sub push_command {
-
     my ( $self, $command ) = @_;
     confess 'must received a Siebel::Srvrmgr::Daemon::Command as parameter'
       unless ( blessed($command) eq 'Siebel::Srvrmgr::Daemon::Command' );
     my $cmds_ref = $self->get_commands();
     push( @{$cmds_ref}, $command );
     $self->set_commands($cmds_ref);    # must trigger the attribute
-
 }
 
 =pod
@@ -686,21 +458,27 @@ sub push_command {
 
 This is the method used to execute commands in srvrmgr program and must be overrided by subclasses of Siebel::Srvrmgr::Daemon.
 Subclasses should invoke L<Moose> C<super> to when doing override because this implementation will apply lock control when appropriate.
+Expects as parameters a L<Siebel::Srvrmgr::Connection>, or it will C<confess> with an exception.
 
 =cut
 
 sub run {
-
-    my $self  = shift;
-    my $class = blessed($self);
-    confess
-      'Only subclasses of Siebel::Srvrmgr::Daemon can executed this method'
-      unless ( defined($class) and ( $self->isa('Siebel::Srvrmgr::Daemon') ) );
+    my ( $self ) = @_;
+    # :TODO:11-12-2016 20:43:19:: wtf is this?
+#    my $class = blessed($self);
+#
+#    if ( defined($class) ) {
+#        confess
+#"Only subclasses of Siebel::Srvrmgr::Daemon can executed this method (received '$class')"
+#          unless ( $self->isa('Siebel::Srvrmgr::Daemon') );
+#    }
+#    else {
+#        confess
+#"Only subclasses of Siebel::Srvrmgr::Daemon can executed this method (received unblessed '$self')";
+#    }
 
     if ( $self->has_lock ) {
-
         $self->_create_lock;
-
     }
 
 }
@@ -718,31 +496,22 @@ See perlport -> Issues -> Newlines for details on this.
 =cut
 
 sub normalize_eol {
-
-    my $self     = shift;
-    my $data_ref = shift;
-
+    my ( $self, $data_ref ) = @_;
     my $ref_type = ref($data_ref);
-
     confess 'data parameter must be an array or scalar reference'
       unless ( ( $ref_type eq 'ARRAY' ) or ( $ref_type eq 'SCALAR' ) );
     my $c_regex = qr/\o{15}?\o{12}/;
 
     if ( $ref_type eq 'ARRAY' ) {
-
         local $/ = "\o{12}";
 
         foreach ( @{$data_ref} ) {
-
             s/$c_regex/\n/g;
-
         }
 
     }
     else {
-
         $$data_ref =~ s/$c_regex/\n/g;
-
     }
 
 }
@@ -753,16 +522,19 @@ sub normalize_eol {
 
 Returns an instance of a L<Siebel::Srvrmgr::ListParser> class.
 
+Optionally, it can receive as parameter a string representing the field delimiter that is expected. If received, 
+the parameter will be used to create the instance.
+
 =cut
 
 sub create_parser {
-    my $self = shift;
+    my ( $self, $delimiter ) = @_;
 
-    if ( $self->get_field_del() ) {
+    if ( defined($delimiter) ) {
         return Siebel::Srvrmgr::ListParser->new(
             {
                 clear_raw       => $self->clear_raw(),
-                field_delimiter => $self->get_field_del()
+                field_delimiter => $delimiter
             }
         );
     }
@@ -773,31 +545,12 @@ sub create_parser {
 
 }
 
-sub _define_params {
-
-    my $self = shift;
-
-    my @params = (
-        $self->get_bin(),        '/e',
-        $self->get_enterprise(), '/g',
-        $self->get_gateway(),    '/u',
-        $self->get_user(),       '/l',
-        $self->get_lang_id()
-
-    );
-
-    push( @params, '/s', $self->get_server() )
-      if ( defined( $self->get_server() ) );
-
-    push( @params, '/k', $self->get_field_del() )
-      if ( defined( $self->get_field_del() ) );
-
 # :WORKAROUND:06/08/2013 21:05:32:: if a perl script will be executed (like for automated testing of this distribution)
 # then the perl interpreter must be part of the command path to avoid calling cmd.exe in Microsoft Windows
-    unshift( @params, $Config{perlpath} ) if ( $self->use_perl() );
-
-    return \@params;
-
+sub _define_params {
+    my ( $self, $params_ref ) = @_;
+    unshift( @{$params_ref}, $Config{perlpath} ) if ( $self->use_perl() );
+    return $params_ref;
 }
 
 =head2 get_lock_file
@@ -807,14 +560,10 @@ Returns the complete path to the lock file as a string.
 =cut
 
 sub get_lock_file {
-
-    my $self = shift;
-
+    my $self     = shift;
     my $filename = blessed($self);
     $filename =~ s/\:{2}/_/g;
-
     return File::Spec->catfile( $self->get_lock_dir, ( $filename . '.lock' ) );
-
 }
 
 our $adm_60070    = qr/^SBL-ADM-60070.*/;
@@ -925,38 +674,28 @@ is just to C<return> true. C<_my_cleanup> is called with a reference of a L<Log:
 =cut
 
 sub DEMOLISH {
-
     my $self = shift;
-
     my $logger = Siebel::Srvrmgr->gimme_logger( blessed($self) );
-
     $logger->info('Terminating daemon: preparing cleanup');
-
     $self->_my_cleanup();
 
     if ( $self->has_lock() ) {
-
         $self->_del_lock();
-
     }
 
     $logger->info('Cleanup is finished');
 
     if ( $logger->is_warn() ) {
-
         $logger->warn('Program termination was forced by ALRM signal')
           if ($SIG_ALARM);
         $logger->warn('Program termination was forced by INT signal')
           if ($SIG_INT);
         $logger->warn('Program termination was forced by PIPE signal')
           if ($SIG_PIPE);
-
     }
 
     $logger->info( ref($self) . ' says bye-bye' ) if ( $logger->is_info() );
-
     Log::Log4perl->remove_logger($logger);
-
 }
 
 =head1 CONSTANTS
@@ -976,15 +715,11 @@ All of them from the L<Fcntl> module.
 =cut
 
 sub _create_lock {
-
     my $self = shift;
-
     my $logger = Siebel::Srvrmgr->gimme_logger( blessed($self) );
-
     my $lock_file = $self->get_lock_file;
 
     if ( -e $lock_file ) {
-
         open( my $in, '<', $lock_file )
           or $logger->logdie("Cannot read $lock_file: $!");
         flock( $in, LOCK_EX | LOCK_NB )
@@ -999,36 +734,28 @@ sub _create_lock {
 
     }
     else {
-
         open( my $out, '>', $lock_file )
           or $logger->logdie("Cannot create $lock_file: $!");
         flock( $out, LOCK_EX | LOCK_NB )
           or $logger->logdie("Could not get exclusive lock on $lock_file: $!");
         print $out $$;
         close($out);
-
     }
 
 }
 
 sub _del_lock {
-
     my $self      = shift;
     my $lock_file = $self->get_lock_file;
-
     my $logger = Siebel::Srvrmgr->gimme_logger( blessed($self) );
 
     if ( -e $lock_file ) {
-
         unlink($lock_file)
           or $logger->logdie("Could not remove $lock_file: $!");
-
     }
     else {
-
         $logger->warn(
             'Could not find lock file to remove before program termination');
-
     }
 
 }
@@ -1105,13 +832,10 @@ sub _define_lock_dir {
 }
 
 sub _setup_commands {
-
     my $self = shift;
 
     foreach my $cmd ( @{ $self->get_commands } ) {
-
         $self->check_cmd( $cmd->get_command() );
-
     }
 
 }
