@@ -4,7 +4,7 @@ package Siebel::Srvrmgr::Daemon::Heavy;
 
 =head1 NAME
 
-Siebel::Srvrmgr::Daemon::Heavy - "heavier" implementation of Siebel::Srvrmgr::Daemon
+Siebel::Srvrmgr::Daemon::Heavy - subclass that reuses srvrmgr program instance for long periods
 
 =head1 SYNOPSIS
 
@@ -12,14 +12,7 @@ Siebel::Srvrmgr::Daemon::Heavy - "heavier" implementation of Siebel::Srvrmgr::Da
 
     my $daemon = Siebel::Srvrmgr::Daemon::Heavy->new(
         {
-            server      => 'servername',
-            gateway     => 'gateway',
-            enterprise  => 'enterprise',
-            user        => 'user',
-            password    => 'password',
-            bin         => 'c:\\siebel\\client\\bin\\srvrmgr.exe',
             time_zone   => 'America/Sao_Paulo',
-            has_lock    => 1, 
             commands    => [
                     Siebel::Srvrmgr::Daemon::Command->new(
                         command => 'load preferences',
@@ -43,12 +36,13 @@ Siebel::Srvrmgr::Daemon::Heavy - "heavier" implementation of Siebel::Srvrmgr::Da
                 ]
         }
     );
+    $daemon->run($connection);
 
 
 =head1 DESCRIPTION
 
 This class extends L<Siebel::Srvmrgr::Daemon>. By "Heavy" you should understand as more complex code to be able to deal with a large number of commands
-of C<srvrmgr>.
+of C<srvrmgr> that will be submitted to a Siebel Enterprise with a short time between them.
 
 This class is indicated to be used in scenarios where several commands need to be executed in a short time interval: it will connect to srvrmgr by using 
 IPC for communication between the processes and once connected, the srvrmgr session will be reused as many times as desired instead of following the
@@ -70,17 +64,18 @@ use Siebel::Srvrmgr::Regexes
   qw(SRVRMGR_PROMPT LOAD_PREF_RESP SIEBEL_ERROR ROWS_RETURNED);
 use Siebel::Srvrmgr::Daemon::Command;
 use POSIX;
-use Data::Dumper;
 use Scalar::Util qw(openhandle);
 use Config;
-use Siebel::Srvrmgr::IPC;
+use Siebel::Srvrmgr::IPC qw(safe_open3);
 use IO::Select;
 use Encode;
 use Carp qw(longmess);
 use Siebel::Srvrmgr;
-our $VERSION = '0.27'; # VERSION
+
+our $VERSION = '0.28'; # VERSION
 
 extends 'Siebel::Srvrmgr::Daemon';
+with 'Siebel::Srvrmgr::Daemon::Connection';
 
 our $SIG_INT   = 0;
 our $SIG_PIPE  = 0;
@@ -94,6 +89,36 @@ our $SIG_ALARM = 0;
 
 This class has additional attributes besides those from parent class.
 
+=head2 maximum_retries
+
+The maximum times this class wil retry to launch a new process of srvrmgr if the previous one failed for any reason. This is intented to implement
+robustness to the process.
+
+=cut
+
+has maximum_retries => (
+    isa     => 'Int',
+    is      => 'ro',
+    reader  => 'get_max_retries',
+    writer  => '_set_max_retries',
+    default => 5
+);
+
+=head2 retries
+
+The number of retries of launching a new srvrmgr process. If this value reaches the value defined for C<maximum_retries>, the instance of Siebel::Srvrmgr::Daemon
+will quit execution returning an error code.
+
+=cut
+
+has retries => (
+    isa     => 'Int',
+    is      => 'ro',
+    reader  => 'get_retries',
+    writer  => '_set_retries',
+    default => 0
+);
+
 =head2 write_fh
 
 A filehandle reference to the C<srvrmgr> STDIN. This is a read-only attribute.
@@ -256,27 +281,20 @@ has srvrmgr_prompt =>
 =cut
 
 sub _add_retry {
-
     my ( $self, $new, $old ) = @_;
 
     # if $old is undefined, this is the first call to run method
     unless ( defined($old) ) {
-
         return 0;
-
     }
     else {
 
         unless ( $new == $old ) {
-
             $self->_set_retries( $self->get_retries() + 1 );
             return 1;
-
         }
         else {
-
             return 0;
-
         }
 
     }
@@ -292,14 +310,19 @@ This methods calls C<clear_pid> just to have a sane setting on C<child_pid> attr
 =cut
 
 sub BUILD {
-
     my $self = shift;
     $self->clear_pid();
-
 }
 
 =pod
 
+=head2 get_retries
+
+Getter for the C<retries> attribute.
+
+=head2 get_max_retries 
+
+Getter for the C<max_retries> attribute.
 
 =head2 clear_pid
 
@@ -352,31 +375,23 @@ Returns the content of the attribute C<params_stack>.
 =cut
 
 override '_setup_commands' => sub {
-
     my $self = shift;
-
     super();
-
     my $cmds_ref = $self->get_commands();
-
     my @cmd;
     my @actions;
     my @params;
 
     foreach my $cmd ( @{$cmds_ref} ) {
-
         push( @cmd,     $cmd->get_command() );
         push( @actions, $cmd->get_action() );
         push( @params,  $cmd->get_params() );
-
     }
 
     $self->_set_cmd_stack( \@cmd );
     $self->_set_action_stack( \@actions );
     $self->_set_params_stack( \@params );
-
     return 1;
-
 };
 
 =pod
@@ -395,38 +410,30 @@ Those operations will be executed in a loop as long the C<check> method from the
 # srvrmgr but the program will hang if there is no output left to be read from srvrmgr.
 
 override 'run' => sub {
-
-    my $self = shift;
-
+    my ($self) = @_;
     super();
-
     my $logger;
     my $temp;
     my $ignore_output = 0;
-
     my ( $read_h, $write_h, $error_h );
 
     unless ( $self->has_pid() ) {
-
-        confess( $self->get_bin()
+        confess( $self->get_conn->get_bin()
               . ' returned un unrecoverable error, aborting execution' )
-          unless ( $self->_create_child() );
+          unless ( $self->_create_child );
 
 # :WORKAROUND:31/07/2013 14:42:33:: must initialize the Log::Log4perl after forking the srvrmgr to avoid sharing filehandles
         $logger = Siebel::Srvrmgr->gimme_logger( blessed($self) );
 
     }
     else {
-
         $logger = Siebel::Srvrmgr->gimme_logger( blessed($self) );
         $logger->info( 'Reusing PID ', $self->get_pid() )
           if ( $logger->is_debug() );
         $ignore_output = 1;
-
     }
 
     $logger->info('Starting run method');
-
     my @input_buffer;
     my $timeout = $self->get_read_timeout;   # avoid multiple method invocations
 
@@ -449,19 +456,16 @@ override 'run' => sub {
 # :WARNING:22-03-2016 23:21:53:: configuration of EOL is obscure but possible in Siebel. The hardcode values might
 # be a problem
 # :TODO:22-03-2016 23:22:29:: add more attributes to take care of it, with default values
-    my $CR = "\o{15}";
-    my $LF = "\o{12}";
+    my $CR          = "\o{15}";
+    my $LF          = "\o{12}";
     my $eol_regex   = qr/$CR$LF$/;
     my $buffer_size = $self->get_buffer_size();
 
     if ( $logger->is_debug() ) {
-
         $logger->debug( 'Setting '
               . $timeout
               . ' seconds for read srvrmgr output time out' );
-
         $logger->debug("sysread buffer size is $buffer_size");
-
         my $assert = 'Input record separator is ';
 
       SWITCH: {
@@ -501,11 +505,9 @@ override 'run' => sub {
                 unless (( defined( $data_ref->{$fh_name}->{bytes} ) )
                     and ( $data_ref->{$fh_name}->{bytes} > 0 ) )
                 {
-
                     $data_ref->{$fh_name}->{bytes} =
                       sysread( $fh, $data_ref->{$fh_name}->{data},
                         $buffer_size );
-
                 }
                 else {
                     $logger->info(
@@ -520,43 +522,34 @@ override 'run' => sub {
                     my $offset =
                       length(
                         Encode::encode_utf8( $data_ref->{$fh_name}->{data} ) );
-
                     $logger->debug("Offset is $offset")
                       if ( $logger->is_debug() );
-
                     $data_ref->{$fh_name}->{bytes} =
                       sysread( $fh, $data_ref->{$fh_name}->{data},
                         $buffer_size, $offset );
-
                 }
 
                 unless ( defined( $data_ref->{$fh_name}->{bytes} ) ) {
-
                     $logger->fatal( 'sysread returned an error: ' . $! );
                     $self->_check_child();
                     $logger->logdie( 'sysreading from '
                           . $data_ref->{$fh_name}->{type}
                           . " returned an unrecoverable error: $!" );
-
                 }
                 else {
 
                     if ( $logger->is_debug() ) {
-
                         $logger->debug( 'Read '
                               . $data_ref->{$fh_name}->{bytes}
                               . ' bytes from '
                               . $data_ref->{$fh_name}->{type} );
-
                     }
 
                     if ( $data_ref->{$fh_name}->{bytes} == 0 ) {
-
                         $logger->warn(
                             'got EOF from ' . $data_ref->{$fh_name}->{type} );
                         $select->remove($fh);
                         next;
-
                     }
 
                     unless ( ( $data_ref->{$fh_name}->{data} =~ $eol_regex )
@@ -756,45 +749,34 @@ override 'run' => sub {
 };
 
 sub _manage_handlers {
-
-    my $self   = shift;
-    my $select = shift;    # IO::Select object
-
+    my ( $self, $select ) = @_;
     my $logger = Siebel::Srvrmgr->gimme_logger( blessed($self) );
 
     # to keep data from both handles while looping over them
     my %data;
-
     my @handlers_order = (qw(STDOUT STDERR));
     my $counter        = 0;
 
     foreach my $fh ( $self->get_read(), $self->get_error() ) {
-
         my $fh_name = fileno($fh);
-
         $data{$fh_name} = {
             type  => $handlers_order[$counter],
             bytes => 0,
             data  => undef
         };
-
         $select->add($fh);
 
         if ( $logger->is_debug() ) {
 
             if ( openhandle($fh) ) {
-
                 $logger->debug(
 "file handler for $counter is available, with fileno = $fh_name "
                 );
-
             }
             else {
-
                 $logger->debug(
 "file handler for $counter is NOT available, with fileno = $fh_name "
                 );
-
             }
 
         }
@@ -808,85 +790,77 @@ sub _manage_handlers {
 }
 
 sub _create_child {
-
-    my $self = shift;
-
+    my ($self) = @_;
     my $logger = Siebel::Srvrmgr->gimme_logger( blessed($self) );
 
     if ( $self->get_retries() >= $self->get_max_retries() ) {
-
         $logger->fatal( 'Maximum retries to spawn srvrmgr reached: '
               . $self->get_max_retries() );
         $logger->warn(
 'Application will exit with an error return code. Please review log for errors'
         );
         exit(1);
+    }
+    my $conn = $self->get_conn;
 
+# :WORKAROUND:09/01/2017 22:52:54:: on Windows is not configured as executable by default
+    if ( $Config{osname} eq 'MSWin32' ) {
+        $logger->logdie(
+            'Cannot find program ' . $conn->get_bin() . ' to execute' )
+          unless ( -e $conn->get_bin() );
+    }
+    else {
+        $logger->logdie(
+            'Cannot find program ' . $conn->get_bin() . ' to execute' )
+          unless ( -e $conn->get_bin() && -x _ );
     }
 
-    $logger->logdie( 'Cannot find program ' . $self->get_bin() . ' to execute' )
-      unless ( -e $self->get_bin() );
-
-    my $params_ref = $self->_define_params();
-
+    my $params_ref = $conn->get_params;
+    $self->_define_params($params_ref);
     my ( $pid, $write_h, $read_h, $error_h ) = safe_open3($params_ref);
 
   # submit the password, avoiding exposing it in the command line as a parameter
-    syswrite $write_h, ( $self->get_password . "\n" );
-
+    syswrite $write_h, ( $conn->get_password . "\n" );
     $self->_set_pid($pid);
     $self->_set_write($write_h);
     $self->_set_read($read_h);
     $self->_set_error($error_h);
 
     if ( $logger->is_debug() ) {
-
         $logger->debug( 'Forked srvrmgr with the following parameters: '
               . join( ' ', @{$params_ref} ) );
         $logger->debug( 'child PID is ' . $pid );
         $logger->debug( 'IPC buffer size is ' . $self->get_buffer_size() );
-
     }
 
     $logger->info('Started srvrmgr');
 
     unless ( $self->_check_child() ) {
-
         return 0;
-
     }
     else {
-
         $self->_set_child_runs(0);
         return 1;
-
     }
 
 }
 
 sub _process_stderr {
-
     exit if ($SIG_INT);
-    my $self     = shift;
-    my $data_ref = shift;
+    my ( $self, $data_ref ) = @_;
 
     my $logger = Siebel::Srvrmgr->gimme_logger( blessed($self) );
 
     if ( defined($$data_ref) ) {
 
         foreach my $line ( split( "\n", $$data_ref ) ) {
-
             exit if ($SIG_INT);
-
             $self->_check_error( $line, 1 );
-
         }
 
     }
     else {
-
         $logger->warn('Received empty buffer to read');
-
     }
 
 }
@@ -895,12 +869,7 @@ sub _process_stdout {
 
 # :TODO      :07/08/2013 15:12:17:: should this be controlled in instances? or should it be global to the class?
     exit if ( $SIG_INT or $SIG_PIPE );
-
-    my $self       = shift;
-    my $data_ref   = shift;
-    my $buffer_ref = shift;
-    my $condition  = shift;
-
+    my ( $self, $data_ref, $buffer_ref, $condition ) = @_;
     my $logger = Siebel::Srvrmgr->gimme_logger( blessed($self) );
 
 # :TODO      :09/08/2013 19:35:30:: review and remove assigning the compiled regexes to scalar (probably unecessary)
@@ -910,20 +879,15 @@ sub _process_stdout {
     $logger->debug("Raw content is [$$data_ref]") if $logger->is_debug();
 
     foreach my $line ( split( "\n", $$data_ref ) ) {
-
         exit if ( $SIG_INT or $SIG_PIPE );
 
         if ( $logger->is_debug() ) {
 
             if ( defined($line) ) {
-
                 $logger->debug("Recovered line [$line]");
-
             }
             else {
-
                 $logger->debug("Recovered line with undefined content");
-
             }
 
         }
@@ -958,21 +922,16 @@ sub _process_stdout {
                     if ( @{$buffer_ref} ) {
 
                         if ( $buffer_ref->[0] eq '' ) {
-
                             $logger->debug("Ignoring output [$line]");
-
                             $condition->set_cmd_sent(0);
                             @{$buffer_ref} = ();
-
                         }
 
                     }
 
                 }
                 elsif ( scalar( @{$buffer_ref} ) < 1 ) {  # no command submitted
-
                     $condition->set_cmd_sent(0);
-
                 }
                 else {
 
@@ -980,15 +939,12 @@ sub _process_stdout {
                         and ( $buffer_ref->[0] eq $self->get_last_cmd() )
                         and $condition->is_cmd_sent() )
                     {
-
                         $condition->set_cmd_sent(0);
-
                     }
 
                 }
 
                 push( @{$buffer_ref}, $line );
-
                 last SWITCH;
 
             }
@@ -1003,9 +959,7 @@ sub _process_stdout {
 }
 
 sub _check_child {
-
-    my $self = shift;
-
+    my $self   = shift;
     my $logger = Siebel::Srvrmgr->gimme_logger( blessed($self) );
 
     if ( $self->has_pid() ) {
@@ -1015,59 +969,43 @@ sub _check_child {
         kill 0, $self->get_pid();
 
         unless ( kill 0, $self->get_pid() ) {
-
             $logger->fatal( $self->get_bin()
                   . " process returned a fatal error: ${^CHILD_ERROR_NATIVE}" );
-
             $logger->fatal( $? . ' child exit status = ' . ( $? >> 8 ) );
-
             $self->close_child($logger);
-
             return 0;
-
         }
         else {
-
             return 1;
-
         }
 
         # try to read immediatly from stderr if possible
         if ( openhandle( $self->get_error() ) ) {
-
             my $error;
-
             my $select = IO::Select->new();
             $select->add( $self->get_error() );
 
             while ( my $fh = $select->can_read( $self->get_read_timeout() ) ) {
-
                 my $buffer;
                 my $read = sysread( $fh, $buffer, $self->get_buffer_size() );
 
                 if ( defined($read) ) {
 
                     if ( $read > 0 ) {
-
                         $error .= $buffer;
                         next;
-
                     }
                     else {
-
                         $logger->debug(
                             'Reached EOF while trying to get error messages');
-
                     }
 
                 }
                 else {
-
                     $logger->warn(
                         'Could not sysread the STDERR from srvrmgr process: '
                           . $! );
                     last;
-
                 }
 
             }    # end of while block
@@ -1076,9 +1014,7 @@ sub _check_child {
 
         }
         else {
-
             $logger->fatal('Error pipe from child is closed');
-
         }
 
         $logger->fatal('Read pipe from child is closed')
@@ -1088,30 +1024,22 @@ sub _check_child {
 
     }    # end of if has_pid
     else {
-
         return 0;
-
     }
 
 }
 
 sub _my_cleanup {
-
-    my $self = shift;
-
+    my $self   = shift;
     my $logger = Siebel::Srvrmgr->gimme_logger( blessed($self) );
 
     if ( $self->has_pid() and ( $self->get_pid() =~ /\d+/ ) ) {
-
         $self->close_child();
-
     }
     else {
 
         if ( $logger->is_info() ) {
-
             $logger->info('No child process to terminate');
-
         }
 
     }
@@ -1121,7 +1049,6 @@ sub _my_cleanup {
 }
 
 sub _submit_cmd {
-
     my ( $self, $cmd ) = @_;
     my $logger = Siebel::Srvrmgr->gimme_logger( blessed($self) );
     my $bytes = syswrite $self->get_write(), "$cmd\n";
@@ -1129,17 +1056,13 @@ sub _submit_cmd {
     if ( defined($bytes) ) {
 
         if ( $logger->is_debug() ) {
-
             $logger->debug("Submitted $cmd, wrote $bytes bytes");
-
         }
 
     }
     else {
-
         $logger->logdie(
             'A failure occurred when trying to submit ' . $cmd . ': ' . $! );
-
     }
 
     return 1;
