@@ -4,7 +4,7 @@ package Siebel::Srvrmgr::Daemon::Light;
 
 =head1 NAME
 
-Siebel::Srvrmgr::Daemon::Light - class for running commmands with Siebel srvrmgr program
+Siebel::Srvrmgr::Daemon::Light - subclass for running commmands with srvrmgr in batch mode
 
 =head1 SYNOPSIS
 
@@ -12,12 +12,8 @@ Siebel::Srvrmgr::Daemon::Light - class for running commmands with Siebel srvrmgr
 
     my $daemon = Siebel::Srvrmgr::Daemon::Light->new(
         {
-            server      => 'servername',
-            gateway     => 'gateway',
-            enterprise  => 'enterprise',
-            user        => 'user',
-            password    => 'password',
-            bin         => 'c:\\siebel\\client\\bin\\srvrmgr.exe',
+            time_zone   => 'America/Sao_Paulo',
+            connection => $my_connection, 
 			commands    => [
 			        Siebel::Srvrmgr::Daemon::Command->new(
                         command => 'load preferences',
@@ -41,6 +37,7 @@ Siebel::Srvrmgr::Daemon::Light - class for running commmands with Siebel srvrmgr
                 ]
         }
     );
+    $daemon->run;
 
 
 =head1 DESCRIPTION
@@ -64,18 +61,19 @@ C<run> available in the C<commands> attribute.
 use Moose 2.0401;
 use namespace::autoclean 0.13;
 use Siebel::Srvrmgr::Daemon::ActionFactory;
-use Siebel::Srvrmgr::ListParser;
 use Siebel::Srvrmgr::Daemon::Command;
-use Config;
 use Carp qw(longmess);
 use File::Temp 0.2304 qw(:POSIX);
 use Data::Dumper;
 use Siebel::Srvrmgr;
 use File::BOM 0.14 qw(:all);
 use Siebel::Srvrmgr::IPC qw(check_system);
-our $VERSION = '0.27'; # VERSION
+use Try::Tiny 0.27;
+our $VERSION = '0.28'; # VERSION
 
 extends 'Siebel::Srvrmgr::Daemon';
+with 'Siebel::Srvrmgr::Daemon::Cleanup';
+with 'Siebel::Srvrmgr::Daemon::Connection';
 
 =pod
 
@@ -123,11 +121,9 @@ Returns the content of the C<input_file> attribute.
 
 =head2 run
 
-This method will try to connect to a Siebel Enterprise through C<srvrmgr> program (if it is the first time the method is invoke) or reuse an already open
-connection to submit the commands and respective actions defined during object creation. The path to the program is check and if it does not exists the 
-method will issue an warning message and immediatly returns false.
+This method will try to connect to a Siebel Enterprise through C<srvrmgr> connection to submit the commands and respective actions defined during object creation.
 
-Those operations will be executed in a loop as long the C<check> method from the class L<Siebel::Srvrmgr::Daemon::Condition> returns true.
+Those operations will be executed in the sequence define until all were executed once.
 
 Beware that Siebel::Srvrmgr::Daemon uses a B<single instance> of a L<Siebel::Srvrmgr::ListParser> class to process the parsing requests, so it is not possible
 to execute L<Siebel::Srvrmgr::Daemon::Command> instances in parallel.
@@ -135,30 +131,32 @@ to execute L<Siebel::Srvrmgr::Daemon::Command> instances in parallel.
 =cut
 
 override 'run' => sub {
-    my $self = shift;
+    my ( $self ) = @_;
     super();
     my $logger = Siebel::Srvrmgr->gimme_logger( blessed($self) );
     $logger->info('Starting run method');
-    my $parser = $self->create_parser();
+    my $parser     = $self->create_parser( $self->get_conn->get_field_del );
+    my $params_ref = $self->get_conn->get_params_pass;
+    $self->_define_params($params_ref);
 
     if ( $logger->is_debug() ) {
-
         $logger->debug( 'Calling system with the following parameters: '
-              . Dumper( $self->_define_params() ) );
+              . Dumper($params_ref) );
         $logger->debug(
             'Commands to be execute are: ' . Dumper( $self->get_commands() ) );
-
     }
 
-    my $ret_code = system( @{ $self->_define_params() } );
+    my $ret_code = system( @{$params_ref} );
     $self->_check_system( ${^CHILD_ERROR_NATIVE}, $ret_code, $? );
     my $in;
-    eval { open_bom( $in, $self->get_output_file(), ':utf8' ) };
 
-    if ($@) {
-        $logger->logdie(
-            'Cannot read ' . $self->get_output_file() . ': ' . $@ );
+    try {
+        open_bom( $in, $self->get_output_file(), ':utf8' );
     }
+    catch {
+        $logger->logdie(
+            'Cannot read ' . $self->get_output_file() . ': ' . $_ );
+    };
 
 # :TODO:22-09-2014 01:32:45:: this might be dangerous if the output is too large
     my @input_buffer = <$in>;
@@ -178,12 +176,10 @@ override 'run' => sub {
             my $total = $self->cmds_vs_tree( $parser->count_parsed() );
 
             if ( $logger->is_debug() ) {
-
                 $logger->debug( 'Total number of parsed items = '
                       . $parser->count_parsed() );
                 $logger->debug( 'Total number of submitted commands = '
                       . scalar( @{ $self->get_commands() } ) );
-
             }
 
             $logger->logdie(
@@ -194,9 +190,7 @@ override 'run' => sub {
             $parser->clear_parsed_tree();
 
             for ( my $i = 0 ; $i < $total ; $i++ ) {
-
-                my $cmd = ( @{ $self->get_commands() } )[$i];
-
+                my $cmd    = ( @{ $self->get_commands() } )[$i];
                 my $action = Siebel::Srvrmgr::Daemon::ActionFactory->create(
                     $cmd->get_action(),
                     {
@@ -204,221 +198,94 @@ override 'run' => sub {
                         params => $cmd->get_params()
                     }
                 );
-
                 $action->do_parsed( $parsed_ref->[$i] );
-
             }
 
         }
         else {
-
             $logger->logdie('Parser did not have a parsed tree after parsing');
-
         }
 
     }
     else {
-
         $logger->debug('buffer is empty');
-
     }
 
     $self->_set_child_runs( $self->get_child_runs() + 1 );
     $logger->debug( 'child_runs = ' . $self->get_child_runs() )
       if ( $logger->is_debug() );
     $logger->info('Exiting run sub');
-
     return 1;
-
 };
 
-=pod
-
-=head2 cmds_vs_tree
-
-This method compares the number of C<commands> defined in a instance of this class with the number of nodes passed as parameter.
-
-If their are equal, the number is returned. If their are different (and there is a problem with the parsed output of srvrmgr) this method
-returns C<undef>.
-
-=cut
-
-sub cmds_vs_tree {
-
-    my $self      = shift;
-    my $nodes_num = shift;
-
-    my $cmds_num = scalar( @{ $self->get_commands() } );
-
-    if ( $cmds_num == $nodes_num ) {
-
-        return $nodes_num;
-
-    }
-    else {
-
-        return;
-
-    }
-
-}
-
 override _my_cleanup => sub {
-
     my $self = shift;
-
     return $self->_del_input_file() && $self->_del_output_file();
-
 };
 
-sub _del_file {
-
-    my $self     = shift;
-    my $filename = shift;
-
-    if ( defined($filename) ) {
-        if ( -e $filename ) {
-
-            my $ret = unlink $filename;
-
-            if ($ret) {
-
-                return 1;
-
-            }
-            else {
-
-                warn "Could not remove $filename: $!";
-                return 0;
-
-            }
-
-        }
-        else {
-
-            warn "File $filename does not exists";
-            return 0;
-
-        }
-    }
-
-}
-
-sub _del_input_file {
-
-    my $self = shift;
-
-    return $self->_del_file( $self->get_input_file() );
-
-}
-
-sub _del_output_file {
-
-    my $self = shift;
-
-    return $self->_del_file( $self->get_output_file() );
-
-}
-
 override _setup_commands => sub {
-
     my $self = shift;
-
     super();
-
     my ( $fh, $input_file ) = tmpnam();
 
     foreach my $cmd ( @{ $self->get_commands() } ) {
-
         print $fh $cmd->get_command(), "\n";
-
     }
 
     close($fh);
-
     $self->_set_input_file($input_file);
-
 };
 
 override _define_params => sub {
-
-    my $self = shift;
-
-    my $params_ref = super();
-
+    my ( $self, $params_ref ) = @_;
+    super();
     $self->_set_output_file( scalar( tmpnam() ) );
-
     push(
         @{$params_ref},
         '/b', '/i', $self->get_input_file(),
         '/o', $self->get_output_file()
     );
-
-# :TODO:10/06/2015 07:09:25 PM:: this will expose the parameter when list the running processes
-    push( @{$params_ref}, '/p', $self->get_password() );
-
     return $params_ref;
-
 };
 
 # :TODO:18-10-2013:arfreitas: this should be done by IPC.pm module?
 sub _manual_check {
-
-    my $self       = shift;
-    my $ret_code   = shift;
-    my $error_code = shift;
-    my $logger     = Siebel::Srvrmgr->gimme_logger( blessed($self) );
+    my ( $self, $ret_code, $error_code ) = @_;
+    my $logger = Siebel::Srvrmgr->gimme_logger( blessed($self) );
 
     if ( $ret_code == 0 ) {
-
         $logger->info(
             'Child process terminate successfully with return code = 0');
-
     }
     else {
-
         $logger->logdie( 'system failed to execute srvrmgr: ' . $error_code );
-
     }
 
 }
 
 sub _check_system {
-
-    my $self        = shift;
-    my $child_error = shift;
-    my $ret_code    = shift;
-    my $error_code  = shift;
+    my ( $self, $child_error, $ret_code, $error_code ) = @_;
 
     my $logger = Siebel::Srvrmgr->gimme_logger( blessed($self) );
-
     my ( $message, $is_error ) = check_system($child_error);
 
     unless ( ( defined($message) ) and ( defined($is_error) ) ) {
-
         $self->_manual_check( $ret_code, $error_code );
-
     }
     else {
 
         if ( defined($is_error) ) {
 
             if ($is_error) {
-
                 $logger->logdie($message);
-
             }
             else {
-
                 $logger->info($message);
-
             }
 
         }
         else {
-
             $logger->info( $message . "Error code is $error_code" );
-
         }
 
     }
@@ -427,12 +294,6 @@ sub _check_system {
 
 =pod
 
-=head1 CAVEATS
-
-This class is still considered experimental and should be used with care.
-
-The C<srvrmgr> program uses buffering, which makes difficult to read the generated output as expected.
-
 =head1 SEE ALSO
 
 =over
