@@ -3,7 +3,7 @@ package Test::Siebel::Srvrmgr::ListParser::OutputFactory;
 use Test::Most;
 use parent 'Test::Siebel::Srvrmgr';
 
-sub constructor : Tests(+13) {
+sub static_methods : Tests(16) {
 
     my $test  = shift;
     my $class = $test->class;
@@ -32,12 +32,31 @@ sub constructor : Tests(+13) {
 
     is( ref($table), 'HASH', 'get_mapping returns an hash ref' );
 
-    my $previous = scalar( keys( %{ $class->get_mapping() } ) );
+    my $total_maps = scalar( keys( %{ $class->get_mapping() } ) );
 
     ok( delete( $table->{list_comp} ),
         'it is ok to remove keys from the hash ref' );
 
-    is( $previous, scalar( keys( %{ $class->get_mapping() } ) ), 'original mapping stays untouched' );
+    is(
+        $total_maps,
+        scalar( keys( %{ $class->get_mapping() } ) ),
+        'original mapping stays untouched'
+    );
+
+    $table = $class->get_mapping();
+
+    note(
+'Validating that "list comps" does not matches other similar commands by mistake'
+    );
+    foreach my $cmd ( ( 'list comp types', 'list comp defs' ) ) {
+        unlike( $cmd, $table->{list_comp}->[1], "list_comp cannot match $cmd" );
+    }
+    note('But matches components with wildcards');
+    like(
+        'list comp EAI%',
+        $table->{list_comp}->[1],
+        'list_comp matches "list comp EAI%'
+    ) or diag(explain($table->{list_comp}));
 
 }
 
